/*
	exporter.cc
*/

#include <fstream>
#include <iostream>
#include <stdexcept>
#include "config.h"
#include "edge.h"
#include "exporter.h"
#include "graph.h"
#include "string.h"
#include "vertex.h"


Exporter *Exporter::make (const String format, const String fname)
{
	if (format == "GraphThing")
		return new GraphThing_Exporter (fname);
	else if (format == "GraphML")
		return new GraphML_Exporter (fname);
	else if (format == "Graphviz")
		return new Graphviz_Exporter (fname);
	else if (format == "Maple")
		return new Maple_Exporter (fname);

	throw std::runtime_error ("Unknown output format!");
}

Exporter::Exporter (const String fname) : filename (fname)
{
	fs.open (filename.c_str (), std::fstream::out);
	if (!fs.is_open ())
		throw std::runtime_error ("Couldn't open file.");
}

Exporter::~Exporter ()
{
	fs.close ();
}

void GraphThing_Exporter::output (const Graph *g)
{
	fs << *g;
}

void GraphML_Exporter::output (const Graph *g)
{
	fs << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
	fs << "<!-- This file created by GraphThing " GT_VERSION
				"   (c) 2001-2004 David Symonds -->\n";
	fs << "<!DOCTYPE graphml SYSTEM \"graphml.dtd\">\n";
	fs << "<graphml>\n";

	fs << "    <key id=\"weight\" for=\"edge\"/>\n";
	fs << "    <graph edgedefault=\"undirected\">\n";
	fs << "\t<desc></desc>\n";
	fs << "\n";

	// Vertices
	Graph::v_const_iterator vit;
	for (vit = g->v_begin (); vit != g->v_end (); ++vit) {
		Vertex *v = *vit;

		// TODO: filter out quotes and other nasties!
		fs << "\t<node id=\"" << v->label << "\"/>\n";
	}
	fs << "\n";

	// Edges
	Graph::e_const_iterator eit;
	for (eit = g->e_begin (); eit != g->e_end (); ++eit) {
		Edge *e = *eit;

		// TODO: filter out quotes and other nasties!
		fs << "\t<edge source=\"" << e->v->label << "\" target=\""
			<< e->w->label << "\"";
		if (e->directed)
			fs << " directed=\"true\"";
		fs << ">\n";
		fs << "\t    <data key=\"weight\">" << e->weight << "</data>\n";
		fs << "\t</edge>\n";
	}

	fs << "    </graph>\n";
	fs << "</graphml>\n";
}

void Graphviz_Exporter::output (const Graph *g)
{
	Graph::e_const_iterator eit;
	bool digraph = false;

	for (eit = g->e_begin (); eit != g->e_end (); ++eit) {
		if ((*eit)->directed) {
			digraph = true;
			break;
		}
	}

	fs << (digraph ? "digraph" : "graph") << " \"G\" {\n";

	// defaults
	fs << "\tnode [\n";
	fs << "\t\tfontname = \"Arial\"\n";
	fs << "\t\tlabel = \"\\N\"\n";
	fs << "\t\tshape = \"circle\"\n";
	fs << "\t\twidth = \"0.5\"\n";
	fs << "\t\theight = \"0.5\"\n";
	fs << "\t\tcolor = \"black\"\n";
	fs << "\t]\n";
	fs << "\tedge [\n";
	fs << "\t\tcolor = \"black\"\n";
	fs << "\t\tweight = \"1\"\n";
	fs << "\t]\n";
	fs << "\n";

	// Vertices are combined with edges in this format
	for (eit = g->e_begin (); eit != g->e_end (); ++eit) {
		Edge *e = *eit;

		fs << "\t\"" << e->v->label << "\" "
			<< (e->directed ? "->" : "--") << " \""
			<< e->w->label << "\"";
		if (e->weight != 1)
			fs << " [weight=\"" << e->weight << "\"]";
		fs << ";\n";
	}

	fs << "}\n";
}

void Maple_Exporter::output (const Graph *g)
{
	// TODO: handle digraphs

	fs << "> with(networks):\n";

	// Vertex set
	fs << "> vertexset:=[";
	Graph::v_const_iterator vit;
	for (vit = g->v_begin (); vit != g->v_end (); ++vit) {
		if (vit != g->v_begin ())
			fs << ",";
		fs << (*vit)->label;	// TODO: handle nasty names
	}
	fs << "];\n";

	// Edge set
	fs << "> edgeset:=[";
	Graph::e_const_iterator eit;
	for (eit = g->e_begin (); eit != g->e_end (); ++eit) {
		Edge *e = *eit;
		if (eit != g->e_begin ())
			fs << ",";
					// TODO: handle nasty names
		fs << "{" << e->v->label << "," << e->w->label << "}";
	}
	fs << "];\n";

	// Edge weights
	fs << "> weightset:=[";
	for (eit = g->e_begin (); eit != g->e_end (); ++eit) {
		if (eit != g->e_begin ())
			fs << ",";
		fs << (*eit)->weight;
	}
	fs << "];\n";

	fs <<	"> new(G):\n"
		"> addvertex(vertexset,G);\n"
		"> addedge(edgeset,weights=weightset,G);\n";
}
