/*
	graph.cc
*/

#include <fstream>
#include <iostream>
#include <map>
#include <stdexcept>
#include <string.h>
#include <time.h>
#include <vector>
#include "config.h"
#include "edge.h"
#include "graph.h"
#include "string.h"
#include "vertex.h"


void Graph::reindex ()
{
	Vmap.erase (Vmap.begin (), Vmap.end ());

	Graph::v_const_iterator vit;
	for (vit = v_begin (); vit != v_end (); ++vit)
		Vmap[(*vit)->label] = *vit;
}

Graph::Graph ()
{
	e_selected_head = 0;
	v_selected_head = 0;
}

Graph::Graph (const Graph &other)
{
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit;
	Vertex *v;

	e_selected_head = 0;
	v_selected_head = 0;

	/* Copy vertices across */
	for (vit = other.v_begin (); vit != other.v_end (); ++vit) {
		v = new Vertex (**vit);
		(*vit)->map_to = v;
		add (v);
	}

	/* Now copy edges */
	for (eit = other.e_begin (); eit != other.e_end (); ++eit) {
		Edge *e = new Edge ((*eit)->v->map_to, (*eit)->w->map_to,
					(*eit)->directed, (*eit)->weight);
		add (e);
	}
}

Graph::~Graph ()
{
	clear ();
}

std::ostream &operator<< (std::ostream &o, const Graph &g)
{
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit;

	/* Info section - empty at the moment */
	o << "info {\n}\n\n";

	/* Dump vertices */
	for (vit = g.v_begin (); vit != g.v_end (); ++vit)
		o << **vit;
	o << '\n';

	/* Dump edges */
	for (eit = g.e_begin (); eit != g.e_end (); ++eit)
		o << **eit;

	return o;
}

Graph *Graph::load (const String &fname, bool &success)
{
	return (load (fname.c_str (), success));
}

Graph *Graph::load (const char *fname, bool &success)
{
	extern Graph *new_graph;	/* in gt-bison.y */
	extern std::fstream *yy_gt_fs;
	extern int yy_gt_parse (void);

	std::fstream fs;

	fs.open (fname, std::fstream::in);
	if (!fs.is_open ()) {
		/* std::cerr << "*** Couldn't open \"" << fname << "\"\n"; */
		return 0;
	}

	/* We don't want anything skipped */
#ifdef USING_FREEBSD
#include <osreldate.h>
#if __FreeBSD_version >= 500035
	fs.setf (std::ios_base::fmtflags (0));
#else
	fs.setf (0);
#endif
#else
	fs.setf (std::ios::fmtflags (0));
#endif

	new_graph = new Graph ();
	yy_gt_fs = &fs;
	success = (yy_gt_parse () == 0);

	fs.close ();

	return new_graph;
}

void Graph::save (const String &fname) const
{
	save (fname.c_str ());
}

void Graph::save (const char *fname) const
{
	std::fstream fs;

	fs.open (fname, std::fstream::out);
	if (!fs.is_open ()) {
		/* std::cerr << "*** Couldn't open \"" << fname << "\"\n"; */
		throw std::runtime_error ("Couldn't open file.");
	}

	fs << *this;
	fs.close ();
}

void Graph::print (const String &fname, bool labels, bool weights) const
{
	print (fname.c_str (), labels, weights);
}

void Graph::print (const char *fname, bool labels, bool weights) const
{
	std::fstream fs;
	int minx, miny, maxx, maxy, len;
	time_t timep;
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit;
	char *str;

	fs.open (fname, std::fstream::out);
	if (!fs.is_open ()) {
		/* std::cerr << "*** Couldn't open \"" << fname << "\"\n"; */
		throw std::runtime_error ("Couldn't open file.");
	}

	minx = miny = 999999;
	maxx = maxy = -1;
	for (vit = v_begin (); vit != v_end (); ++vit) {
		if (minx > (*vit)->x)
			minx = (*vit)->x;
		if (maxx < (*vit)->x)
			maxx = (*vit)->x;
		if (miny > (*vit)->y)
			miny = (*vit)->y;
		if (maxy < (*vit)->y)
			maxy = (*vit)->y;
	}
	if (V.size () < 1) {
		minx = miny = 10;
		maxx = maxy = 200;
	}
	minx -= 10;
	miny -= (labels ? 10 : 20);
	maxx += (labels ? 20 : 10);
	maxy += 20;

	timep = time (0);
	str = ctime (&timep);
	len = strlen (str);
	if (str[len - 1] == '\n')
		str[len - 1] = '\0';	// remove trailing newline

	fs <<	"%%!PS-Adobe-2.0\n"
		"%%%%Title: " << fname << "\n"
		"%%%%Creator: GraphThing " GT_VERSION
			"   (c) 2001-2004 David Symonds\n"
		"%%%%CreationDate: " << str << "\n"
		"%%%%Pages: 1\n"
		"%%%%Origin: 0 0\n"
		"%%%%BoundingBox: " << minx << ' ' << miny << ' ' <<
					maxx << ' ' << maxy << "\n\n";

	fs <<	"/Helvetica findfont 12 scalefont setfont\n\n";

	/* Vertex macro */
	fs <<	"%% Args: label, xcenter, ycenter\n"
		"/vertex {\n"
		"\t" << maxy << " exch sub\n"
		"\t1 index 1 index\n"
		"\tnewpath 5 0 360 arc fill\n";
	if (labels)
		fs << "\texch 10 add exch 10 add moveto show\n";
	else
		fs << "\tpop\n";
	fs << "} bind def\n\n";

	/* Arrow head macros */
	fs <<	"%% Arrow head stuff\n"
		"/ArrowHeadSize 15 def\n"
		"%% Args: x1 y1 x2 y2  (arrowhead at x2,y2; from x1,y1)\n"
		"%% Out: x1 y1 x2' y2' (draw line to x2', y2')\n"
		"/arrowhead {\n"
		"\t1 index 4 index sub\n"
		"\t1 index 4 index sub\n"
		"\texch atan\n"
		"\n"
		"\tArrowHeadSize -.8 mul\n"
		"\tdup\n"
		"\t2 index cos mul 4 index add\n"
		"\texch\n"
		"\t2 index sin mul 3 index add\n"
		"\n"
		"\t5 2 roll\n"
		"\n"
		"\tgsave\n"
		"\t\t3 1 roll\n"
		"\t\ttranslate\n"
		"\t\trotate\n"
		"\t\tnewpath\n"
		"\t\t0 0 moveto\n"
		"\t\tArrowHeadSize dup neg exch .25 mul\n"
		"\t\t2 copy lineto\n"
		"\t\tArrowHeadSize -.8 mul 0\n"
		"\t\t2 copy\n"
		"\t\t6 4 roll\n"
		"\t\tneg curveto\n"
		"\t\tclosepath fill\n"
		"\tgrestore\n"
		"} bind def\n\n";

	/* Edge macros */
	fs <<	"%% Return point on line, given by t = [0, 1]\n"
		"%% Args: x1 y1 x2 y2 t\n"
		"%% Out: xt yt\n"
		"/linepositiondict 5 dict def\n"
		"/lineposition {\n"
		"\tlinepositiondict begin\n"
		"\t\t/t exch def\n"
		"\t\t/y2 exch def /x2 exch def\n"
		"\t\t/y1 exch def /x1 exch def\n"
		"\t\tx2 x1 sub t mul x1 add\n"
		"\t\ty2 y1 sub t mul y1 add\n"
		"\tend\n"
		"} bind def\n\n";
	fs <<	"%% Args: label, startx, starty, endx, endy\n"
		"/edge {\n"
		"\t" << maxy << " exch sub 3 index 3 index " << maxy <<
							" exch sub\n"
		"\t1 index 1 index moveto\n"
		"\t3 index 3 index lineto stroke\n";
	if (labels)
		fs <<	"\t0.5 lineposition\n"
			"\texch 5 add exch 5 add moveto pop pop show\n";
	else
		fs << "\tpop pop pop pop pop pop pop\n";
	fs <<	"} bind def\n\n";
	fs <<	"%% Args: label, startx, starty, endx, endy\n"
		"/dir_edge {\n"
		"\t" << maxy << " exch sub 3 index 3 index " << maxy <<
							" exch sub\n"
		"\t3 index 3 index arrowhead\n"
		"\t3 index 3 index moveto\n"
		"\tlineto stroke\n";
	if (labels)
		fs <<	"\t0.5 lineposition\n"
			"\texch 5 add exch 5 add moveto pop pop show\n";
	else
		fs << "\tpop pop pop pop pop pop pop\n";
	fs <<	"} bind def\n\n";

	/* We dump the edges first so that the vertices will be printed
	 * "on top" of the edge ends
	 */

	/* Dump edges first */
	fs << "%% Edges\n";
	for (eit = e_begin (); eit != e_end (); ++eit) {
		if (weights)
			fs << "(" << (*eit)->weight << ") ";
		else
			fs << "() ";
		fs << (*eit)->v->x << ' ' << (*eit)->v->y << ' ' <<
			(*eit)->w->x << ' ' << (*eit)->w->y <<
			((*eit)->directed ? " dir_edge\n" : " edge\n");
	}
	fs << '\n';

	/* Now dump the vertices */
	fs << "%% Vertices\n";
	for (vit = v_begin (); vit != v_end (); ++vit)
		fs << '(' << (*vit)->label << ") " << (*vit)->x << ' ' <<
						(*vit)->y << " vertex\n";

	fs << "\n\nshowpage\n";

	fs.close ();
}

void Graph::clear ()
{
	for (; !E.empty (); E.pop_back ())
		delete E.back ();
	for (; !V.empty (); V.pop_back ())
		delete V.back ();
	Vmap.erase (Vmap.begin (), Vmap.end ());
}

void Graph::add (Edge *e)
{
	Edge *e2;

	if (are_adjacent (e->v, e->w, e->directed)) {
		delete e;
		return;		/* fail silently */
	}
	if (e->directed && (e2 = find (e->w, e->v, true))) {
		// A directed edge already exists going the other way.
		// We simply make it undirected, and forget about this edge.

		e2->directed = false;
		delete e;
		return;
	}
	e->selected = false;
	E.push_back (e);
	e->v->hook (e);
	e->w->hook (e);
}

void Graph::add (Vertex *v)
{
	if (v->label.empty () || find (v->label))
		v->label = unique_label ();
	v->selected = false;
	v->selection_colour = 0;
	V.push_back (v);
	Vmap[v->label] = v;
}

void Graph::rename (Vertex *v, const String &new_label)
{
	Vmap.erase (v->label);
	v->label = new_label;
	Vmap[v->label] = v;
}

Vertex *Graph::find (const String &label) const
{
	std::map<const String, Vertex *>::const_iterator it;

	it = Vmap.find (label);
	if (it == Vmap.end ())
		return 0;
	return it->second;
}

Vertex *Graph::find (int x, int y) const
{
	Graph::v_const_iterator vit;
	int r2 = Graph::vertex_radius * Graph::vertex_radius;

	for (vit = v_begin (); vit != v_end (); ++vit) {
		int dx = x - (*vit)->x, dy = y - (*vit)->y;
		if ((dx * dx + dy * dy) < r2)
			return *vit;
	}

	return 0;
}

Edge *Graph::find (const Vertex *v1, const Vertex *v2, bool dir) const
{
	Graph::e_const_iterator eit;

	if (!dir && (v1->degree () > v2->degree ())) {
		const Vertex *tmp = v1;
		v1 = v2;
		v2 = tmp;
	}

	if (!dir) {
		for (eit = v1->e_begin (); eit != v1->e_end (); ++eit) {
			if ((*eit)->incident_to (v2))
				return *eit;
		}
	} else {
		for (eit = v1->e_begin (); eit != v1->e_end (); ++eit) {
			if ((*eit)->v != v1)
				continue;
			if ((*eit)->w == v2)
				return *eit;
		}
	}
	return 0;
}

Edge *Graph::findE (int x, int y) const
{
	Graph::e_const_iterator eit;
	double r2 = Graph::vertex_radius * Graph::vertex_radius;
	double best_dist = 0;		// actually, best distance squared
	Edge *best_edge = 0;

	// find straight-line edge closest to (x,y)
	for (eit = e_begin (); eit != e_end (); ++eit) {
		double x1, y1, x2, y2, s2, num, d2;
		x1 = (*eit)->v->x;
		y1 = (*eit)->v->y;
		x2 = (*eit)->w->x;
		y2 = (*eit)->w->y;
		if (x1 < x2) {
			if ((x < x1) || (x2 < x))
				continue;
		} else {
			if ((x < x2) || (x1 < x))
				continue;
		}
		if (y1 < y2) {
			if ((y < y1) || (y2 < y))
				continue;
		} else {
			if ((y < y2) || (y1 < y))
				continue;
		}
		s2 = (x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1);
		if (s2 < 0.01)
			continue;	// degenerate edge
		num = (y2 - y1) * (x - x1) - (x2 - x1) * (y - y1);
		d2 = num * num / s2;
		if ((d2 < best_dist) || !best_edge) {
			best_edge = *eit;
			best_dist = d2;
		}
	}

	// still only accept it if within a vertex of (x,y)
	if (best_dist <= r2)
		return best_edge;
	return 0;
}

void Graph::remove (Edge *e)
{
	Graph::e_iterator eit;

	if (e->selected)
		unselect (e);

	e->v->unhook (e);
	e->w->unhook (e);

	for (eit = e_begin (); eit != e_end (); ++eit) {
		if (**eit == *e) {
			E.erase (eit);
			delete e;
			break;
		}
	}
}

void Graph::remove (Vertex *v)
{
	Graph::e_iterator eit;
	Graph::v_iterator vit;

	if (v->selected)
		unselect (v);

	/* Remove edges incident to v */
	for (eit = v->e_begin (); v->degree (); )
		remove (*eit);

	/* Remove v */
	Vmap.erase (v->label);
	for (vit = v_begin (); vit != v_end (); ++vit) {
		if (*vit == v) {
			V.erase (vit);
			delete v;
			break;
		}
	}
}

void Graph::select (Edge *e)
{
	if (e->selected)
		return;

	e->selected = true;

	e->next = e_selected_head;
	e_selected_head = e;
}

void Graph::select (Vertex *v)
{
	if (v->selected)
		return;

	v->selected = true;

	v->next = v_selected_head;
	v_selected_head = v;
}

void Graph::unselect (Edge *e)
{
	if (!e->selected)
		return;

	e->selected = false;

	if (e_selected_head == e)
		e_selected_head = e->next;
	else {
		Edge *prev = e_selected_head;
		while (prev->next != e)
			prev = prev->next;
		prev->next = e->next;
	}
	e->next = 0;
}

void Graph::unselect (Vertex *v)
{
	if (!v->selected)
		return;

	v->selected = false;
	v->selection_colour = 0;

	if (v_selected_head == v)
		v_selected_head = v->next;
	else {
		Vertex *prev = v_selected_head;
		while (prev->next != v)
			prev = prev->next;
		prev->next = v->next;
	}
	v->next = 0;
}

void Graph::unselect_all ()
{
	while (e_selected_head)
		unselect (e_selected_head);
	while (v_selected_head)
		unselect (v_selected_head);
}

String Graph::unique_label (int level) const
{
	String buf ("A");
	int i;

	for (i = 0; i < level; ++i)
		buf += "-dup";

	do {
		if (!find (buf))
			return buf;
		buf[0]++;
		//buf.set (0, buf[0] + 1);
	} while (buf[0] <= 'Z');

	/* Failed - try next level */
	return (unique_label (level + 1));
}

bool Graph::are_adjacent (const Vertex *v1, const Vertex *v2, bool dir) const
{
	Graph::e_const_iterator eit;

	if (!dir && (v1->degree () > v2->degree ())) {
		const Vertex *tmp = v1;
		v1 = v2;
		v2 = tmp;
	}

	if (!dir) {
		for (eit = v1->e_begin (); eit != v1->e_end (); ++eit) {
			if ((*eit)->incident_to (v2))
				return true;
		}
	} else {
		for (eit = v1->e_begin (); eit != v1->e_end (); ++eit) {
			if ((*eit)->directed) {
				if (((*eit)->v != v1))
					continue;
				if ((*eit)->w == v2)
					return true;
			} else if ((*eit)->incident_to (v2))
				return true;
		}
	}

	return false;
}

void Graph::identify (Vertex *v1, Vertex *v2)
{
	Edge *e;
	Graph::e_const_iterator eit;

	unselect (v1);
	unselect (v2);

	/* Swap vertices, so least-degree vertex is v1 */
	if (v1->degree () > v2->degree ()) {
		Vertex *tmp = v1;
		v1 = v2;
		v2 = tmp;
	}

	/* Remove any edge between them */
	while ((e = find (v1, v2, false)))
		remove (e);

	/* Now move all edges to v2, then destroy v1 */
	for (eit = v1->e_begin (); eit != v1->e_end (); ++eit) {
		Vertex *valt;

		valt = v1->opposite (*eit);
		if ((*eit)->directed) {
			if ((*eit)->v == v1) {
				if (are_adjacent (v2, valt, true))
					continue;
				add (new Edge (v2, valt, true));
			} else {
				if (are_adjacent (valt, v2, true))
					continue;
				add (new Edge (valt, v2, true));
			}
		} else {
			if (are_adjacent (valt, v2, false))
				continue;
			add (new Edge (valt, v2));
		}
	}

	remove (v1);
}

// Return a subgraph induced by the marked (mark != 0) vertices
Graph *Graph::subgraph_marked () const
{
	Graph *ret;
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit;

	ret = new Graph;
	for (vit = v_begin (); vit != v_end (); ++vit)
		if ((*vit)->mark)
			ret->add (new Vertex (**vit));
	for (eit = e_begin (); eit != e_end (); ++eit) {
		Edge *e = *eit;
		if (e->v->mark && e->w->mark)
			ret->add (new Edge (ret->find (e->v->label),
						ret->find (e->w->label)));
	}

	return ret;
}

Graph *Graph::line_graph () const
{
	Graph *ret = new Graph ();
	Graph::e_const_iterator eit, eit2;
	Graph::v_const_iterator vit;

	for (eit = e_begin (); eit != e_end (); ++eit) {
		Edge *e = *eit;
		//String s = e->v->label + "," + e->w->label;
		Vertex *v = new Vertex ("", (e->v->x + e->w->x) / 2,
						(e->v->y + e->w->y) / 2);
		e->mark = (int) v;
		ret->add (v);
	}

	for (vit = v_begin (); vit != v_end (); ++vit) {
		Vertex *v = *vit;
		for (eit = v->e_begin (); eit != v->e_end (); ++eit) {
			Vertex *lg_1 = (Vertex *) (*eit)->mark;
			for (eit2 = eit + 1; eit2 != v->e_end (); ++eit2) {
				Vertex *lg_2 = (Vertex *) (*eit2)->mark;
				if (!are_adjacent (lg_1, lg_2))
					ret->add (new Edge (lg_1, lg_2));
			}
		}
	}

	return ret;
}

Graph *Graph::flattened () const
{
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit;
	Graph *g = new Graph ();

	/* Copy vertices across */
	for (vit = v_begin (); vit != v_end (); ++vit)
		g->add (new Vertex (**vit));

	/* Now copy edges */
	for (eit = e_begin (); eit != e_end (); ++eit) {
		Edge *e = new Edge (	g->find ((*eit)->v->label),
					g->find ((*eit)->w->label),
					false, (*eit)->weight);
		g->add (e);
	}

	return g;
}

Graph::e_iterator Graph::e_begin ()
{
	return E.begin ();
}

Graph::e_iterator Graph::e_end ()
{
	return E.end ();
}

Graph::v_iterator Graph::v_begin ()
{
	return V.begin ();
}

Graph::v_iterator Graph::v_end ()
{
	return V.end ();
}

Graph::e_const_iterator Graph::e_begin () const
{
	return E.begin ();
}

Graph::e_const_iterator Graph::e_end () const
{
	return E.end ();
}

Graph::v_const_iterator Graph::v_begin () const
{
	return V.begin ();
}

Graph::v_const_iterator Graph::v_end () const
{
	return V.end ();
}

Vertex *Graph::operator[] (unsigned int i)
{
	return V[i];
}

const Vertex *Graph::operator[] (unsigned int i) const
{
	return V[i];
}

unsigned int Graph::order () const
{
	return V.size ();
}

unsigned int Graph::num_edges () const
{
	return E.size ();
}
