/*
	gui.h
*/

#ifndef __GUI_H__
#define __GUI_H__

#include <gtk--/dialog.h>
#include <gtk--/drawingarea.h>
#include <gtk--/window.h>
#include <vector>
#include "matrix.h"
#include "string.h"
#include "undo.h"


class Canvas;
class Graph;
class Edge;
class Vertex;

// Avoid inclusion of large header files
namespace Gtk {
	class CheckMenuItem;
	class Entry;
	class Menu;
	class MenuBar;
	class MenuItem;
	class OptionMenu;
	class ProgressBar;
	class SpinButton;
	class Statusbar;
	class Table;
	class Toolbar;
	namespace Toolbar_Helpers {
		class RadioElem;
	}
}


class GUI : public Gtk::Window
{
private:
	Graph *graph;
	UndoStack undoStack;

	Canvas *canvas;
	Gtk::MenuItem *edit_Undo;
	Gtk::CheckMenuItem *view_Labels, *view_Weights;
	Gtk::Statusbar *statusBar;
	Gtk::ProgressBar *progressBar;
	Gdk_GC gc;

	/* All callbacks are in gui_cb.cc */
	void cb_File_Load ();
	void cb_File_Print ();
	void cb_File_Save ();
	void cb_Edit_Undo ();
	void cb_Edit_SelectAll ();
	void cb_Edit_SelectNone ();
	void cb_Edit_InvertSelectionVertices ();
	void cb_Edit_InvertSelectionEdges ();
	void cb_Edit_InvertSelectionAll ();
	void cb_View ();
	void cb_Graph_Clear ();
	void cb_Graph_Complement ();
	void cb_Graph_LineGraph ();
	void cb_Graph_Subgraph ();
	void cb_Graph_Find_ShortestPath ();
	void cb_Graph_Find_BFS ();
	void cb_Graph_Find_DFS ();
	void cb_Graph_Find_MST ();
	void cb_Graph_Properties_Connectivity ();
	void cb_Graph_Properties_Eulericity ();
	void cb_Graph_Statistics_AdjacencyMatrix ();
	void cb_Graph_Statistics_DegreeSequence ();
	void cb_Graph_Statistics_Diameter ();
	void cb_Graph_Statistics_Radius ();
	void cb_Graph_Statistics_ChromaticNumber ();
	void cb_Graph_Statistics_ChromaticIndex ();
	void cb_Graph_Statistics_ChromaticPolynomial ();
	void cb_Prefab_Complete ();
	void cb_Prefab_CompleteBipartite ();
	void cb_Prefab_Cycle ();
	void cb_Prefab_Gear ();
	void cb_Prefab_Hanoi ();
	void cb_Prefab_Ladder ();
	void cb_Prefab_Lattice ();
	void cb_Prefab_Null ();
	void cb_Prefab_Star ();
	void cb_Prefab_Wheel ();
	void cb_Prefab_Petersen ();
	void cb_Prefab_Platonic_Tetrahedral ();
	void cb_Prefab_Platonic_Cubical ();
	void cb_Prefab_Platonic_Octahedral ();
	void cb_Prefab_Platonic_Dodecahedral ();
	void cb_Prefab_Platonic_Icosahedral ();
	void cb_Help_About ();

	// Split mode
	Gtk::Toolbar *toolbar;
	Gtk::Toolbar_Helpers::RadioElem *vmode, *emode;
	const static int VERTEX_MODE = 1, EDGE_MODE = 2;
	int editing_mode;
	void cb_Change_Mode (int mode);
	int cb_Change_Mode_Hack ();


	Gtk::MenuBar *genMenuBar ();
	void setUndoText (String description);

protected:
	virtual int delete_event_impl (GdkEventAny *ev);
	virtual int key_press_event_impl (GdkEventKey *ev);
	virtual void realize_impl ();
	gint quitter ();

public:
	GUI ();
	~GUI ();

	void undoableAction (String description);
	void load_graph (const char *fname);
	void msg (const String &title, const String &s);

	void set_progress (double frac);	// argument in [0, 1]
};


class MsgDialog : public Gtk::Dialog
{
protected:
	virtual int delete_event_impl (GdkEventAny *ev);

public:
	MsgDialog (const String &title, const String &s);
};


class ParamDialog : public Gtk::Dialog
{
protected:
	void ok_click ();

	virtual int delete_event_impl (GdkEventAny *ev);
	virtual void get_data () = 0;

public:
	bool cancelled;

	ParamDialog (const String &title = "Prefab Parameter");
};


class IntParamDialog : public ParamDialog
{
protected:
	Gtk::SpinButton *sp;

	virtual void get_data ();

public:
	int param;

	IntParamDialog (const String &s, int min, int initial, int max = 50);
	IntParamDialog (const String &title, const String &s, int min,
					int initial, int max = 50);
};


class IntIntParamDialog : public ParamDialog
{
protected:
	Gtk::SpinButton *sp1, *sp2;

	virtual void get_data ();

public:
	int param1;
	int param2;

	IntIntParamDialog (const String &s, int min1, int initial1,
						int min2, int initial2);
};


class StringParamDialog : public ParamDialog
{
protected:
	Gtk::Entry *entry;

	virtual void get_data ();

public:
	String param;

	StringParamDialog (const String &s, const String &initial);
	StringParamDialog (const String &title, const String &s,
						const String &initial);
};


class EdgeParamDialog : public ParamDialog
{
protected:
	Gtk::SpinButton *sp;
	Gtk::OptionMenu *dir;

	virtual void get_data ();

	const static int max_edge_weight = 200;

public:
	int weight;
	int direction;	// -1=left, 0=neither, 1=right

	EdgeParamDialog (const Edge *e);

	void setLeft () { direction = -1; }
	void setNeither () { direction = 0; }
	void setRight () { direction = 1; }
};


class Canvas : public Gtk::DrawingArea
{
private:
	Gdk_Colormap cmap;
	Gdk_Color white, black, red;
	std::vector<Gdk_Color> selection_colours;
	Gdk_Font bold_font, normal_font;
	Gdk_GC gc;
	Gdk_Window win;
	GUI *gui;
	Graph **gg;
	bool do_labels, do_weights;
	int motion_last_x, motion_last_y;
	int size_x, size_y;

	void draw (Edge *e);
	void draw (Vertex *v);

	void cb_Properties ();

protected:
	virtual int button_press_event_impl (GdkEventButton *ev);
	virtual int expose_event_impl (GdkEventExpose *ev);
	virtual int motion_notify_event_impl (GdkEventMotion *ev);
	virtual void realize_impl ();
	virtual void size_allocate_impl (GtkAllocation *al);

public:
	bool vertex_mode;

	Canvas (GUI *the_gui, Graph **g);

	void redraw ();
	void set_param (bool labels, bool weights);
};


// Implemented in gui_matrix.cc
class MatrixDialog : public Gtk::Dialog
{
private:
	std::vector<Matrix> matrices;
	std::vector<String> labels;
	Gtk::SpinButton *sp;
	Gtk::Table *tbl;

	void update_view ();

protected:
	virtual int delete_event_impl (GdkEventAny *ev);

public:
	MatrixDialog (const String &title, const Matrix &_mat,
							const Graph &g);
};

#endif	/* __GUI_H__ */
