/*
	gui_cb.cc
*/

#include <gtk--/checkmenuitem.h>
#include <gtk--/main.h>
#include <gtk--/statusbar.h>
#include <gtk--/toolbar.h>
#include <set>
#include <sstream>
#include <vector>
#include "config.h"
#include "edge.h"
#include "exporter.h"
#include "factory.h"
#include "fancyfileselection.h"
#include "graph.h"
#include "gui.h"
#include "lang.h"
#include "matrix.h"
#include "polynomial.h"
#include "string.h"
#include "undo.h"
#include "vertex.h"



void GUI::cb_File_Load ()
{
	FancyFileSelection fs (_("Load File"));
	bool res;

	Gtk::Main::run ();

	if (fs.cancelled)
		return;

	Graph *g = graph->load (fs.get_filename (), res);
	fs.hide ();
	if (!g) {
		msg ("Loading failed", "Loading failed.");
		return;
	}
	if (!res)
		msg (_("Warning"), _("WARNING:\n\n"
			"The loading partially failed;\n"
			"this graph may be incomplete."));

	undoableAction ("Load graph");
	delete graph;
	graph = g;

	canvas->redraw ();
}

void GUI::cb_File_Print ()
{
	FancyFileSelection fs (_("Print to File"), "*.ps");

	Gtk::Main::run ();

	if (!fs.cancelled)
		graph->print (fs.get_filename (), view_Labels->is_active (),
						view_Weights->is_active ());
}

void GUI::cb_File_Save ()
{
	FancyFileSelection fs (_("Save File"), "*.gt", true, true);

	Gtk::Main::run ();

	if (fs.cancelled)
		return;

	Exporter *exp = Exporter::make (fs.filetype, fs.get_filename ());
	exp->output (graph);
	delete exp;
}

void GUI::cb_Edit_Undo ()
{
	if (undoStack.empty ())
		return;
	//std::cerr << "DEBUG: Undoing '" << undoStack.top ().getMessage () << "'...\n";
	delete graph;
	graph = undoStack.pop ().getGraph ();
	canvas->redraw ();
	if (undoStack.empty ()) {
		edit_Undo->set_sensitive (false);
		setUndoText ("");
	} else
		setUndoText (undoStack.top ().getMessage ());
}

void GUI::cb_Edit_SelectAll ()
{
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit;

	for (eit = graph->e_begin (); eit != graph->e_end (); ++eit)
		graph->select (*eit);
	for (vit = graph->v_begin (); vit != graph->v_end (); ++vit)
		graph->select (*vit);

	canvas->redraw ();
}

void GUI::cb_Edit_SelectNone ()
{
	graph->unselect_all ();
	canvas->redraw ();
}

void GUI::cb_Edit_InvertSelectionVertices ()
{
	Graph::v_const_iterator vit;

	for (vit = graph->v_begin (); vit != graph->v_end (); ++vit) {
		if ((*vit)->selected)
			graph->unselect (*vit);
		else
			graph->select (*vit);
	}

	canvas->redraw ();
}

void GUI::cb_Edit_InvertSelectionEdges ()
{
	Graph::e_const_iterator eit;

	for (eit = graph->e_begin (); eit != graph->e_end (); ++eit) {
		if ((*eit)->selected)
			graph->unselect (*eit);
		else
			graph->select (*eit);
	}

	canvas->redraw ();
}

void GUI::cb_Edit_InvertSelectionAll ()
{
	cb_Edit_InvertSelectionVertices ();
	cb_Edit_InvertSelectionEdges ();
}

void GUI::cb_View ()
{
	canvas->set_param (view_Labels->is_active (),
				view_Weights->is_active ());
}

void GUI::cb_Graph_Clear ()
{
	undoableAction ("Clear graph");
	graph->clear ();
	canvas->redraw ();
}

void GUI::cb_Graph_Complement ()
{
	Graph *orig = graph;
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit, vit2;

	undoableAction ("Complement graph");
	graph = new Graph;
	for (vit = orig->v_begin (); vit != orig->v_end (); ++vit)
		graph->add (new Vertex (**vit));
	// Method: create a new complete graph, then remove the edges that were
	//	in the original. FIXME: speed this up!
	for (vit = graph->v_begin (); vit != graph->v_end (); ++vit)
		for (vit2 = vit + 1; vit2 != graph->v_end (); ++vit2)
			graph->add (new Edge (*vit, *vit2));
	for (eit = orig->e_begin (); eit != orig->e_end (); ++eit) {
		Vertex *v1, *v2;
		v1 = graph->find ((*eit)->v->label);
		v2 = graph->find ((*eit)->w->label);
		graph->remove (graph->find (v1, v2));
	}

	delete orig;
	canvas->redraw ();
}

void GUI::cb_Graph_LineGraph ()
{
	undoableAction ("Line graph");
	Graph *orig = graph;
	graph = orig->line_graph ();
	delete orig;
	canvas->redraw ();
}

void GUI::cb_Graph_Subgraph ()
{
	Graph *orig = graph;
	Graph::v_const_iterator vit;

	if (!graph->v_selected_head) {
		msg (_("Subgraph"), _("At least one vertex must be selected "
						"to induce a subgraph."));
		return;
	}

	undoableAction ("Induced subgraph");

	// Mark selection
	for (vit = graph->v_begin (); vit != graph->v_end (); ++vit) {
		Vertex *v = *vit;
		v->mark = v->selected ? 1 : 0;
	}

	graph = orig->subgraph_marked ();
	delete orig;
	canvas->redraw ();
}

void GUI::cb_Graph_Find_ShortestPath ()
{
	Vertex *v1, *v2, *walk;
	Graph::e_const_iterator eit;
	int tot;

	v2 = graph->v_selected_head;
	if (!v2 || !v2->next || v2->next->next) {
		msg (_("Shortest Path"), _("Exactly two vertices must be "
				"selected to find a shortest path."));
		return;
	}
	v1 = v2->next;

	statusBar->push (1, _("Finding shortest path..."));

	graph->mark_shortest_path (v1, v2);

	/* Select shortest path */
	tot = v2->mark;
	graph->unselect_all ();
	graph->select (v2);
	walk = v2;
	while (walk != v1) {
		Edge *e = 0;
		Vertex *alt = 0;

		for (eit = walk->e_begin (); eit != walk->e_end (); ++eit) {
			e = *eit;
			if (e->selected)
				continue;
			alt = walk->opposite (e);
			if (walk->mark == (alt->mark + e->weight))
				break;
		}
		if (eit == walk->e_end ())
			break;		/* Uh, oh! */
		graph->select (e);
		graph->select (alt);
		walk = alt;
	}

	if (tot < 0) {
		/* If there is no path at all, leave original vertices
		 * selected
		 */
		graph->unselect_all ();
		graph->select (v1);
		graph->select (v2);
	}

	canvas->redraw ();

	/* FIXME: Work out how to do this using std::ostringstream, but
	 *	still keeping the gettext i18n ability.
	 */
	if (tot >= 0) {
		char buf[32];
		g_snprintf (buf, 32, _("Shortest path is %i."), tot);
		String s (buf);
		msg (_("Shortest Path"), s);
	} else
		msg (_("Shortest Path"), _("No path found."));

	statusBar->pop (1);
}

void GUI::cb_Graph_Find_BFS ()
{
	Vertex *v;

	v = graph->v_selected_head;
	if (!v || v->next) {
		msg (_("Breadth-First Search"), _("Exactly one vertex "
					"must be selected to perform a "
					"Breadth-First Search."));
		return;
	}

	statusBar->push (1, _("Performing BFS..."));

	String m ("BFS:\n\n");
	graph->bfs (v, m);

	msg (_("Breadth-First Search"), m);

	statusBar->pop (1);
}

void GUI::cb_Graph_Find_DFS ()
{
	Vertex *v;

	v = graph->v_selected_head;
	if (!v || v->next) {
		msg (_("Depth-First Search"), _("Exactly one vertex must "
			"be selected to perform a Depth-First Search."));
		return;
	}

	statusBar->push (1, _("Performing DFS..."));

	String m ("DFS:\n\n");
	graph->dfs (v, m);

	msg (_("Depth-First Search"), m);

	statusBar->pop (1);
}

void GUI::cb_Graph_Find_MST ()
{
	std::set<Edge *> spanning_tree;
	std::set<Edge *>::const_iterator it;

	statusBar->push (1, _("Finding Minimum Spanning Tree..."));

	graph->minimum_spanning_tree (spanning_tree);
	graph->unselect_all ();
	for (it = spanning_tree.begin (); it != spanning_tree.end (); ++it) {
		graph->select (*it);
		graph->select ((*it)->v);
		graph->select ((*it)->w);
	}
	canvas->redraw ();

	statusBar->pop (1);
}

void GUI::cb_Graph_Properties_Connectivity ()
{
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit;
	bool undir, conn, strong_conn = false;

	statusBar->push (1, _("Testing connectivity..."));

	undir = graph->is_undirected ();

	conn = graph->is_connected ();
	if (conn && !undir)
		strong_conn = graph->is_strongly_connected ();
	if (conn) {
		if (undir)
			msg (_("Connectivity"), _("Graph is connected."));
		else if (!strong_conn)
			msg (_("Connectivity"), _("Graph is weakly connected."));
		else
			msg (_("Connectivity"), _("Graph is strongly connected."));
	} else {
		msg (_("Connectivity"), _("Graph is not connected."));

		/* Select marked vertices and edges */
		graph->unselect_all ();
		for (eit = graph->e_begin (); eit != graph->e_end (); ++eit)
			if ((*eit)->mark)
				graph->select (*eit);
		for (vit = graph->v_begin (); vit != graph->v_end (); ++vit)
			if ((*vit)->mark)
				graph->select (*vit);
		canvas->redraw ();
	}

	statusBar->pop (1);
}

void GUI::cb_Graph_Properties_Eulericity ()
{
	bool euler, semi;

	statusBar->push (1, _("Determining Eulericity..."));

	String tour ("\n(");
	graph->eulericity (euler, semi, tour);
	tour += ")";

	if (euler)
		msg (_("Eulericity"), _("Graph is Eulerian.") + tour);
	else if (semi)
		msg (_("Eulericity"), _("Graph is Semi-Eulerian.") + tour);
	else
		msg (_("Eulericity"), _("Graph is neither Eulerian nor "
						"Semi-Eulerian."));

	statusBar->pop (1);
}

void GUI::cb_Graph_Statistics_AdjacencyMatrix ()
{
	MatrixDialog md (_("Adjacency Matrix"),
				graph->adjacency_matrix (), *graph);
	Gtk::Main::run ();

	/* cout << graph->adjacency_matrix (); */
}

void GUI::cb_Graph_Statistics_DegreeSequence ()
{
	std::vector<int> seq;
	std::vector<int>::iterator it1, it2;
	Graph::v_const_iterator vit;

	if (graph->order () < 1) {
		std::ostringstream m;
		m << _("Degree Sequence") << ":\n\n(none)";
		msg (_("Degree Sequence"), m.str ());
		return;
	}

	for (vit = graph->v_begin (); vit != graph->v_end (); ++vit)
		seq.push_back ((*vit)->degree ());

	/* boring bubble sort */
	for (it1 = seq.begin (); (it1 + 1) != seq.end (); ++it1)
	for (it2 = it1 + 1; it2 != seq.end (); ++it2) {
		if (*it1 < *it2) {
			int tmp = *it1;
			*it1 = *it2;
			*it2 = tmp;
		}
	}

	std::ostringstream m;
	m << (_("Degree Sequence")) << ":\n\n";

	for (it1 = seq.begin (); it1 != seq.end (); ++it1) {
		if (it1 == seq.begin ())
			m << *it1;
		else
			m << ", " << *it1;
	}

	msg (_("Degree Sequence"), m.str ());
}

void GUI::cb_Graph_Statistics_Diameter ()
{
	std::ostringstream m;

	statusBar->push (1, _("Determining diameter..."));

	m << "diam(G) = " << graph->diameter (true);
	canvas->redraw ();

	msg (_("Diameter"), m.str ());

	statusBar->pop (1);
}

void GUI::cb_Graph_Statistics_Radius ()
{
	std::ostringstream m;

	statusBar->push (1, _("Determining radius..."));

	m << "rad(G) = " << graph->radius (true);
	canvas->redraw ();

	msg (_("Radius"), m.str ());

	statusBar->pop (1);
}

void GUI::cb_Graph_Statistics_ChromaticNumber ()
{
	std::ostringstream m;

	statusBar->push (1, _("Computing chromatic number..."));

	int chi = graph->chromatic_number ();

	graph->unselect_all ();
	while (chi >= 2) {		// WARNING: Abuse of loop construct!
		if (!graph->try_colouring (chi))
			break;
		Graph::v_iterator vit;
		for (vit = graph->v_begin (); vit != graph->v_end (); ++vit) {
			graph->select (*vit);
			(*vit)->selection_colour = (*vit)->mark;
		}
		break;
	}
	// TODO: mention something if we failed to colour properly!

	canvas->redraw ();
	m << "chi(G) = " << chi;
	msg (_("Chromatic Number"), m.str ());

	set_progress (-1);
	statusBar->pop (1);
}

void GUI::cb_Graph_Statistics_ChromaticIndex ()
{
	std::ostringstream m;

	statusBar->push (1, _("Computing chromatic index..."));

	Graph *lg = graph->line_graph ();
	int chi = lg->chromatic_number ();
	delete lg;

	m << "chi'(G) = " << chi;
	msg (_("Chromatic Index"), m.str ());

	set_progress (-1);
	statusBar->pop (1);
}

void GUI::cb_Graph_Statistics_ChromaticPolynomial ()
{
	std::ostringstream m;

	statusBar->push (1, _("Computing chromatic polynomial..."));

	Polynomial p = graph->chromatic_polynomial ();

	m << _("Chromatic Polynomial") << ":\n\n" << p;
	msg (_("Chromatic Polynomial"), m.str ());

	set_progress (-1);
	statusBar->pop (1);
}

void GUI::cb_Prefab_Complete ()
{
	statusBar->push (1, _("Prefab..."));
	IntParamDialog ipd (_("Complete:"), 1, 5);
	Gtk::Main::run ();
	if (ipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::K (ipd.param);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_CompleteBipartite ()
{
	statusBar->push (1, _("Prefab..."));
	IntIntParamDialog iipd (_("Complete Bipartite:"), 1, 3, 1, 3);
	Gtk::Main::run ();
	if (iipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::K (iipd.param1, iipd.param2);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Cycle ()
{
	statusBar->push (1, _("Prefab..."));
	IntParamDialog ipd (_("Cycle:"), 1, 5);
	Gtk::Main::run ();
	if (ipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::C (ipd.param);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Gear ()
{
	statusBar->push (1, _("Prefab..."));
	IntParamDialog ipd (_("Gear:"), 3, 5);
	Gtk::Main::run ();
	if (ipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::G (ipd.param);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Hanoi ()
{
	statusBar->push (1, _("Prefab..."));
	IntParamDialog ipd (_("Hanoi:"), 1, 3);
	Gtk::Main::run ();
	if (ipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::H (ipd.param);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Ladder ()
{
	statusBar->push (1, _("Prefab..."));
	IntParamDialog ipd (_("Ladder:"), 1, 4);
	Gtk::Main::run ();
	if (ipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::L (ipd.param);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Lattice ()
{
	statusBar->push (1, _("Prefab..."));
	IntIntParamDialog iipd (_("Lattice:"), 1, 4, 1, 4);
	Gtk::Main::run ();
	if (iipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::Lattice (iipd.param1, iipd.param2);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Null ()
{
	statusBar->push (1, _("Prefab..."));
	IntParamDialog ipd (_("Null:"), 1, 3);
	Gtk::Main::run ();
	if (ipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::N (ipd.param);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Star ()
{
	statusBar->push (1, _("Prefab..."));
	IntParamDialog ipd (_("Star:"), 1, 5);
	Gtk::Main::run ();
	if (ipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::S (ipd.param);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Wheel ()
{
	statusBar->push (1, _("Prefab..."));
	IntParamDialog ipd (_("Wheel:"), 1, 5);
	Gtk::Main::run ();
	if (ipd.cancelled) {
		statusBar->pop (1);
		return;
	}

	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::W (ipd.param);
	canvas->redraw ();
	statusBar->pop (1);
}

void GUI::cb_Prefab_Petersen ()
{
	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::Petersen ();
	canvas->redraw ();
}

void GUI::cb_Prefab_Platonic_Tetrahedral ()
{
	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::Tetrahedral ();
	canvas->redraw ();
}

void GUI::cb_Prefab_Platonic_Cubical ()
{
	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::Cubical ();
	canvas->redraw ();
}

void GUI::cb_Prefab_Platonic_Octahedral ()
{
	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::Octahedral ();
	canvas->redraw ();
}

void GUI::cb_Prefab_Platonic_Dodecahedral ()
{
	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::Dodecahedral ();
	canvas->redraw ();
}

void GUI::cb_Prefab_Platonic_Icosahedral ()
{
	undoableAction ("Make prefab");
	delete graph;
	graph = Factory::Icosahedral ();
	canvas->redraw ();
}

#ifdef ENABLE_NLS
#include <gtk--/pixmap.h>
#include <gtk--/tooltips.h>
#include "logo_sc.xpm"
#endif
void GUI::cb_Help_About()
{
	std::ostringstream msg;

	statusBar->push (1, _("About GraphThing"));
	msg << "G r a p h T h i n g\n\n";
	msg << _("Version") << " " << GT_VERSION << "\n\n";
	msg << "(c) 2001-2004 David Symonds\n\n";
	msg << _(
		"This program is free software; you can redistribute\n"
		"it and/or modify it under the terms of the GNU\n"
		"General Public License, version 2, as published by\n"
		"the Free Software Foundation.\n\n"

		"This program is distributed in the hope that it will\n"
		"be useful, but WITHOUT ANY WARRANTY; without even\n"
		"the implied warranty of MERCHANTABILITY or FITNESS\n"
		"FOR A PARTICULAR PURPOSE. See the GNU General\n"
		"Public License for more details.");

	MsgDialog m (_("About..."), msg.str ());

#ifdef ENABLE_NLS
	if (translator->get_language () == SwedishChef) {
		Gtk::Box_Helpers::BoxList &blist =
					m.get_action_area ()->children ();
		Gtk::Button *butt = (Gtk::Button *)
					(*(blist.begin ()))->get_widget ();
		butt->remove ();
		butt->set_usize (-1, -1);
		Gtk::Pixmap *pix = manage (new Gtk::Pixmap (logo_sc_xpm));
		butt->add (*pix);

		Gtk::Tooltips *tt = manage (new Gtk::Tooltips ());
		tt->set_tip (*butt, "Bork Bork Bork!");

		m.show_all ();
	}
#endif

	Gtk::Main::run ();

	statusBar->pop (1);
}

//#include <gtk--.h>
void GUI::cb_Change_Mode (int mode)
{
	// This is a bit of a kludge. When you click on a RadioElem in a
	// toolbar, you get both callbacks called - first the unselected one,
	// then the selected one. Also, if you click on an already-selected
	// button, it just does that callback. The workaround that is used here
	// is to just set the mode to whatever we get told - it is a bit of
	// "double-setting", but it works.

	editing_mode = mode;

	statusBar->pop (1);
	if (mode == VERTEX_MODE)
		statusBar->push (1, _("Ready (Vertex Mode)"));
	else
		statusBar->push (1, _("Ready (Edge Mode)"));

	canvas->vertex_mode = (mode == VERTEX_MODE);

	// TODO: depress the correct buttons (mainly for keyboard mode switches)
	//Gtk::Toolbar_Helpers::ToolList &tl = toolbar->tools ();
	//Gtk::Toolbar_Helpers::Tool *tool = tl[1];
	//Gtk::Widget *widget = tool->get_widget (),
	//		*content = tool->get_content ();
	//std::cerr << Gtk::RadioButton::isA (widget) << "\n";
	//std::cerr << Gtk::Pixmap::isA (content) << "\n";
}

int GUI::cb_Change_Mode_Hack ()
{
	cb_Change_Mode (VERTEX_MODE);

	return 0;	// just so Gtk::Main::timeout can call this
}
